import codecs
import json
import os
from datetime import datetime
from pathlib import Path
from typing import Iterable


def make_session_id() -> str:
    return f"{datetime.now():%y%m%d%H%M%S}"


def read_json(path: Path | os.PathLike[str] | str) -> list | dict:
    with codecs.open(path, "r", encoding="utf8") as file:
        return json.load(file)


def try_parse_json(text: str | None) -> str | None:
    # taken from agi_med_common
    if not isinstance(text, str):
        return None
    if not text:
        return None
    if text[0] not in "{[":
        return None
    if text[-1] not in "}]":
        return None
    try:
        return json.loads(text)
    except Exception:
        return None


def try_parse_int(text: str) -> int | None:
    try:
        return int(text)
    except (ValueError, TypeError):
        return None


def try_parse_float(text: str) -> float | None:
    try:
        return float(text)
    except (ValueError, TypeError):
        return None


def try_parse_bool(v: str | bool) -> bool:
    if isinstance(v, bool):
        return v
    return v.lower() in ("yes", "true", "t", "1")


def pretty_line(text: str, cut_count: int = 100) -> str:
    if len(text) > 100:
        text_cut = text[:cut_count]
        size = len(text)
        text_pretty = f"{text_cut}..(total {size} characters)"
    else:
        text_pretty = text
    text_pretty = text_pretty.replace("\n", "\\n")
    return text_pretty


def first_nonnull(obj: Iterable):
    for elem in obj:
        if elem:
            return elem
    return None



__version__ = "5.1.2"

from .models import (
    MTRSLabelEnum,
    ChatItem,
    InnerContextItem,
    OuterContextItem,
    ReplicaItem,
    Chat,
    Context,
    ChatMessage,
    AIMessage,
    HumanMessage,
    MiscMessage,
)
from .models.widget import Widget
from .file_storage import FileStorage, ResourceId
from .models import DiagnosticsXMLTagEnum, MTRSXMLTagEnum, DoctorChoiceXMLTagEnum
from .utils import make_session_id, read_json, try_parse_json, try_parse_int, try_parse_float, pretty_line
from .validators import ExistingPath, ExistingFile, ExistingDir, StrNotEmpty, SecretStrNotEmpty, Prompt, Message
from .xml_parser import XMLParser
from .parallel_map import parallel_map
from .models.tracks import TrackInfo, DomainInfo



from pydantic import Field

from ._base import _Base


class TrackInfo(_Base):
    track_id: str = Field(alias="TrackId")
    name: str = Field(alias="Name")
    domain_id: str = Field(alias="DomainId")


class DomainInfo(_Base):
    domain_id: str = Field(alias="DomainId")
    name: str = Field(alias="Name")



from typing import List, Tuple

from agi_med_common.models import Chat, ChatMessage, DomainInfo, TrackInfo
from pydantic import BaseModel


Value = str
Interpretation = str
ResourceId = str


class ChatManagerAPI:
    def get_domains(self, language_code: str, client_id: str) -> List[DomainInfo]:
        raise NotImplementedError

    def get_tracks(self, language_code: str, client_id: str) -> List[TrackInfo]:
        raise NotImplementedError

    def get_response(self, chat: Chat, request_id: str = "") -> List[ChatMessage]:
        raise NotImplementedError


class TextGeneratorAPI:
    def process(self, chat: Chat, request_id: str = "") -> str:
        raise NotImplementedError


class ContentInterpreterRemoteResponse(BaseModel):
    interpretation: str
    resource_fname: str
    resource: bytes


class ContentInterpreterRemoteAPI:
    def interpret_remote(
        self,
        kind: str,
        query: str,
        resource: bytes,
        chat: Chat | None = None,
        request_id: str = "",
    ) -> ContentInterpreterRemoteResponse:
        raise NotImplementedError


class ClassifierAPI:
    def get_values(self) -> List[Value]:
        raise NotImplementedError

    def evaluate(self, chat: Chat, request_id: str = "") -> Value:
        raise NotImplementedError


class CriticAPI:
    def evaluate(self, text: str, chat: Chat | None = None, request_id: str = "") -> float:
        raise NotImplementedError


class ContentInterpreterAPI:
    def interpret(
        self, kind: str, query: str, resource_id: str = "", chat: Chat | None = None, request_id: str = ""
    ) -> Tuple[Interpretation, ResourceId | None]:
        raise NotImplementedError


class TextProcessorAPI:
    def process(self, text: str, chat: Chat | None = None, request_id: str = "") -> str:
        raise NotImplementedError
